@@ -1,103 +1,80 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2015-2020 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
 
 """
 module to convert from dx file (hdf5) to nexus tomo compliant .nx (hdf5)
 """
 
-__authors__ = [
-    "H. Payno",
-]
-__license__ = "MIT"
-__date__ = "18/05/2021"
+from __future__ import annotations
 
-
-from typing import Union
-from tomoscan.io import HDF5File
-from silx.io.utils import get_data
-from silx.io.url import DataUrl
-from nxtomomill.utils import ImageKey
-from nxtomomill.converter.baseconverter import BaseConverter
-from silx.io.utils import h5py_read_dataset
-from nxtomomill.utils.hdf5 import EntryReader
-from nxtomomill.utils.hdf5 import DatasetReader
-from nxtomomill.converter.version import version as converter_version
-from nxtomomill.nexus.nxdetector import FieldOfView
-from nxtomomill.utils import FrameAppender
-from nxtomomill.io.config import DXFileConfiguration
-from tomoscan.unitsystem import metricsystem
 import logging
-import numpy
-import h5py
 import os
+import pint
 
+import h5py
+import numpy
+from silx.io.url import DataUrl
+from silx.io.utils import get_data, h5py_read_dataset
+from tomoscan.io import HDF5File
+from nxtomo.nxobject.nxdetector import FieldOfView, ImageKey
+from nxtomo.utils.frameappender import FrameAppender
+
+from nxtomomill.converter.baseconverter import BaseConverter
+from nxtomomill.converter.version import version as converter_version
+from nxtomomill.io.config import DXFileConfiguration
+
+from nxtomomill.utils.hdf5 import DatasetReader, EntryReader
+from silx.utils.deprecation import deprecated
+
+_ureg = pint.get_application_registry()
 _logger = logging.getLogger(__name__)
 
 
+__all__ = ["from_dx_to_nx", "from_dx_config_to_nx"]
+
+
+# dev note: to ensure backward compatibility we needed to keep from_dx_to_nx. And we created from_dx_config_to_nx to handle the configuration file.
+# but with time we expect to remove `from_dx_to_nx` and them rename from_dx_config_to_nx to `from_dx_to_nx` in order to get something coherent.
+@deprecated(
+    replacement="from_dx_config_to_nx",
+    since_version="0.13",
+    reason="Some configuration dedicated classes exists for tuning the configuration directly instead of provided n parameter to the function.",
+)
 def from_dx_to_nx(
     input_file: str,
-    output_file: Union[str, None] = None,
+    output_file: str | None = None,
     file_extension: str = ".nx",
     duplicate_data: bool = True,
-    input_entry="/",
-    output_entry="entry0000",
-    scan_range=(0.0, 180.0),
+    input_entry: str = "/",
+    output_entry: str = "entry0000",
+    scan_range: tuple = (0.0, 180.0),
     pixel_size=(None, None),
-    field_of_view=None,
-    distance=None,
-    overwrite=True,
-    energy=None,
+    field_of_view: FieldOfView | str | None = None,
+    distance: float | None = None,
+    overwrite: bool = True,
+    energy: float | None = None,
 ) -> tuple:
     """
 
-    :param str input_file: dxfile to convert
-    :param str output_file: output file to save
-    :param str file_extension: file extension to give if the output file is
-                               not provided.
-    :param bool duplicate_data: if True frames will be duplicated. Otherwise
-                                we will create (relative) link to the input
-                                file.
-    :param str input_entry: path to the HDF5 group to convert. For now it looks
-                            each file can only contain one dataset. Just to
-                            insure any future compatibility if it evolve with
-                            time.
-    :param str output_entry: path to store the NxTomo created.
-    :param tuple scan_range: tuple of two elements with the minimum scan range.
-                             projections are expected to be taken with equal
-                            angular spaces.
-    :param tuple pixel_size: pixel size can be provided (in meter and as
-                             x_pizel_size, y_pixel_size)
+    :param input_file: dxfile to convert
+    :param output_file: output file to save
+    :param file_extension: file extension to give if the output file is
+        not provided.
+    :param duplicate_data: if True frames will be duplicated. Otherwise
+        we will create (relative) link to the input file.
+    :param input_entry: path to the HDF5 group to convert. For now it looks
+        each file can only contain one dataset. Just to ensure any future
+        compatibility if it evolve with time.
+    :param output_entry: path to store the NxTomo created.
+    :param scan_range: tuple of two elements with the minimum scan range.
+        Projections are expected to be taken with equal angular spaces.
+    :param pixel_size: pixel size can be provided (in meter and as
+        x_pizel_size, y_pixel_size)
     :param field_of_view: field of view
-    :type field_of_view: None, str or FieldOfView
     :param distance: sample / detector distance in meter
-    :type distance: None or float
-    :param bool overwrite: if True and if the entry already exists in the
-                           output file then will overwrite it.
+    :param overwrite: if True and if the entry already exists in the
+        output file then will overwrite it.
     :return: tuple of (output_file, entry) created. For now the list should
-             contain at most one of those tuple
-    :rtype: tuple
+        contain at most one of those tuple
     """
     configuration = DXFileConfiguration(input_file=input_file, output_file=output_file)
     configuration.file_extension = file_extension
@@ -107,9 +84,23 @@ def from_dx_to_nx(
     configuration.scan_range = scan_range
     configuration.pixel_size = pixel_size
     configuration.field_of_view = field_of_view
-    configuration.distance = distance
+    configuration.sample_detector_distance = distance
     configuration.overwrite = overwrite
     configuration.energy = energy
+    return from_dx_config_to_nx(configuration=configuration)
+
+
+def from_dx_config_to_nx(
+    configuration: DXFileConfiguration,
+):
+    """
+    Convert from dxfile to NXtomo.
+    Dark and flats will be store at the beginning and we consider they are
+    take at start so rotation angle will set to scan_range[0].
+    Projection rotation angle will be interpolated from scan_range and
+    with equality space distribution.
+    We do not expect any alignment projection.
+    """
     converter = _DxFileToNxConverter(configuration=configuration)
     return converter.convert()
 
@@ -128,11 +119,11 @@ class _DxFileToNxConverter(BaseConverter):
     We do not expect any alignment projection.
     """
 
-    DEFAULT_DISTANCE_VALUE = 1.0 * metricsystem.MetricSystem.METER.value
+    DEFAULT_SAMPLE_DETECTOR_DISTANCE_VALUE: pint.Quantity = 1.0 * _ureg.meter
 
-    DEFAULT_PIXEL_VALUE = 1.0 * metricsystem.MetricSystem.MICROMETER.value
+    DEFAULT_PIXEL_VALUE: pint.Quantity = 1.0 * _ureg.micrometer
 
-    DEFAULT_BEAM_ENERGY = 1.0 * metricsystem.EnergySI.KILOELECTRONVOLT.value
+    DEFAULT_BEAM_ENERGY: pint.Quantity = 1.0 * _ureg.keV
 
     def __init__(
         self,
@@ -142,7 +133,7 @@ class _DxFileToNxConverter(BaseConverter):
         if not len(self.scan_range) == 2:
             raise ValueError("scan_range expects to be a tuple with two elements")
 
-        input_file = os.path.realpath(self.input_file)
+        input_file = os.path.abspath(self.input_file)
         if self.output_file is None:
             input_file_basename, _ = os.path.splitext(input_file)
             if not self._configuration.file_extension.startswith("."):
@@ -155,7 +146,7 @@ class _DxFileToNxConverter(BaseConverter):
                 + self._configuration.file_extension,
             )
         else:
-            output_file = os.path.realpath(self.output_file)
+            output_file = os.path.abspath(self.output_file)
         self._configuration.output_file = output_file
         if self.input_entry == "/":
             self._configuration._input_entry = ""
@@ -165,8 +156,8 @@ class _DxFileToNxConverter(BaseConverter):
             fov = self.field_of_view
             if isinstance(fov, str):
                 fov = fov.title()
-            self._configuration.field_of_view = FieldOfView.from_value(fov)
-        self._configuration.distance = self.distance
+            self._configuration.field_of_view = FieldOfView(fov)
+        self._configuration.sample_detector_distance = self.distance
         self._configuration.overwrite = self.overwrite
 
         self._n_frames = 0
@@ -208,15 +199,15 @@ class _DxFileToNxConverter(BaseConverter):
         return self._configuration.overwrite
 
     @property
-    def distance(self) -> Union[float, None]:
-        return self._configuration.distance
+    def distance(self) -> float | None:
+        return self._configuration.sample_detector_distance
 
     @property
-    def field_of_view(self) -> Union[FieldOfView, None]:
+    def field_of_view(self) -> FieldOfView | None:
         return self._configuration.field_of_view
 
     @property
-    def energy(self) -> Union[float, None]:
+    def energy(self) -> float | None:
         return self._configuration.energy
 
     @property
@@ -229,7 +220,6 @@ class _DxFileToNxConverter(BaseConverter):
 
         :return: tuple of (output_file, entry) created. For now the list should
                  contain at most one of those tuple
-        :rtype: tuple
         """
         with HDF5File(self.output_file, mode="a") as h5f:
             if self.output_entry in h5f:
@@ -281,7 +271,7 @@ class _DxFileToNxConverter(BaseConverter):
             instrument_grp = root_grp.require_group("instrument")
             detector_grp = instrument_grp.require_group("detector")
             detector_grp["x_pixel_size"] = x_pixel_size
-            detector_grp["x_pixel_size"].attrs["unit"] = "m"
+            detector_grp["x_pixel_size"].attrs["units"] = "m"
         #  y pixel size
         if y_pixel_size is None:
             try:
@@ -298,7 +288,7 @@ class _DxFileToNxConverter(BaseConverter):
             instrument_grp = root_grp.require_group("instrument")
             detector_grp = instrument_grp.require_group("detector")
             detector_grp["y_pixel_size"] = y_pixel_size
-            detector_grp["y_pixel_size"].attrs["unit"] = "m"
+            detector_grp["y_pixel_size"].attrs["units"] = "m"
         #  field of view
         if self.field_of_view is not None:
             with HDF5File(self.output_file, mode="a") as h5f:
@@ -309,12 +299,16 @@ class _DxFileToNxConverter(BaseConverter):
         #  distance
         if self.distance is None:
             try:
-                self._configuration.distance = self._read_distance()
+                self._configuration.sample_detector_distance = (
+                    self._read_sample_detector_distance()
+                )
             except Exception:
-                self._configuration.distance = None
+                self._configuration.sample_detector_distance = None
         # set default distance value
         if self.distance is None:
-            self._configuration.distance = self.DEFAULT_DISTANCE_VALUE
+            self._configuration.sample_detector_distance = (
+                self.DEFAULT_SAMPLE_DETECTOR_DISTANCE_VALUE
+            )
             _logger.warning(
                 "No detector / sample distance found or provided. "
                 "Set the it  the default value"
@@ -323,8 +317,8 @@ class _DxFileToNxConverter(BaseConverter):
             root_grp = h5f.require_group(self.output_entry)
             instrument_grp = root_grp.require_group("instrument")
             detector_grp = instrument_grp.require_group("detector")
-            detector_grp["distance"] = self.distance
-            detector_grp["distance"].attrs["unit"] = "m"
+            detector_grp["distance"] = self.distance.to(_ureg.meter)
+            detector_grp["distance"].attrs["units"] = str(_ureg.meter)
         # energy
         if self.energy is None:
             try:
@@ -339,10 +333,8 @@ class _DxFileToNxConverter(BaseConverter):
         with HDF5File(self.output_file, mode="a") as h5f:
             root_grp = h5f.require_group(self.output_entry)
             beam_grp = root_grp.require_group("beam")
-            beam_grp["incident_energy"] = (
-                self.energy / metricsystem.EnergySI.KILOELECTRONVOLT.value
-            )
-            beam_grp["incident_energy"].attrs["unit"] = "kev"
+            beam_grp["incident_energy"] = self.energy.to(_ureg.keV)
+            beam_grp["incident_energy"].attrs["units"] = str(_ureg.keV)
 
         #  count_time
         self._copy_count_time()
@@ -365,9 +357,7 @@ class _DxFileToNxConverter(BaseConverter):
             count_time = h5py_read_dataset(dataset)
             if count_time == "Error: Unknown Attribute":
                 _logger.info(
-                    "Count time not stored on {}@{}".format(
-                        self.input_entry, self.input_file
-                    )
+                    f"Count time not stored on {self.input_entry}@{self.input_file}"
                 )
             else:
                 try:
@@ -379,15 +369,11 @@ class _DxFileToNxConverter(BaseConverter):
                             detector_grp["count_time"] = count_time
                     else:
                         _logger.warning(
-                            "exposure period and data frame have an incoherent size ({} vs {})".format(
-                                len(count_time), self._n_frames
-                            )
+                            f"exposure period and data frame have an incoherent size ({len(count_time)} vs {self._n_frames})"
                         )
                 except Exception as e:
                     _logger.error(
-                        "Failed to get 'count_time' / 'exposure_period'. reason is {}".format(
-                            e
-                        )
+                        f"Failed to get 'count_time' / 'exposure_period'. reason is {e}"
                     )
 
         with EntryReader(self.input_root_url) as input_entry:
@@ -408,9 +394,7 @@ class _DxFileToNxConverter(BaseConverter):
         try:
             data_dark = get_data(self._data_darks_url)
         except _PathDoesNotExistsInExchange:
-            _logger.warning(
-                "No darks found in {}@{}".format(self.input_entry, self.input_file)
-            )
+            _logger.warning(f"No darks found in {self.input_entry}@{self.input_file}")
         else:
             image_key.extend([ImageKey.DARK_FIELD.value] * len(data_dark))
             image_key_control.extend([ImageKey.DARK_FIELD.value] * len(data_dark))
@@ -423,9 +407,7 @@ class _DxFileToNxConverter(BaseConverter):
         try:
             data_flat = get_data(self._data_flats_url)
         except _PathDoesNotExistsInExchange:
-            _logger.warning(
-                "No flats found in {}@{}".format(self.input_entry, self.input_file)
-            )
+            _logger.warning(f"No flats found in {self.input_entry}@{self.input_file}")
         else:
             image_key.extend([ImageKey.FLAT_FIELD.value] * len(data_flat))
             image_key_control.extend([ImageKey.FLAT_FIELD.value] * len(data_flat))
@@ -467,7 +449,7 @@ class _DxFileToNxConverter(BaseConverter):
             detector_grp["image_key_control"] = image_key_control
             sample_grp = root_grp.require_group("sample")
             sample_grp["rotation_angle"] = rotation_angle
-            sample_grp["rotation_angle"].attrs["unit"] = "degree"
+            sample_grp["rotation_angle"].attrs["units"] = str(_ureg.degree)
             self._path_nxtomo_attrs(root_grp)
 
     def _path_nxtomo_attrs(self, root_grp):
@@ -500,6 +482,8 @@ class _DxFileToNxConverter(BaseConverter):
         try:
             with DatasetReader(self._data_darks_url) as dark_dataset:
                 n_dark = dark_dataset.shape[0]
+            # FIXME: avoid keeping some file open. not clear why this is needed
+            dark_dataset = None
             FrameAppender(
                 self._data_darks_url,
                 self.output_file,
@@ -509,9 +493,7 @@ class _DxFileToNxConverter(BaseConverter):
             ).process()
         except Exception:
             _logger.error(
-                "No darks found in {} or unable to add them to the dataset".format(
-                    self._data_darks_url.path()
-                )
+                f"No darks found in {self._data_darks_url.path()} or unable to add them to the dataset"
             )
         else:
             image_key.extend([ImageKey.DARK_FIELD.value] * n_dark)
@@ -522,7 +504,8 @@ class _DxFileToNxConverter(BaseConverter):
         try:
             with DatasetReader(self._data_flats_url) as flat_dataset:
                 n_flat = flat_dataset.shape[0]
-
+            # FIXME: avoid keeping some file open. not clear why this is needed
+            flat_dataset = None
             FrameAppender(
                 self._data_flats_url,
                 self.output_file,
@@ -532,9 +515,7 @@ class _DxFileToNxConverter(BaseConverter):
             ).process()
         except Exception:
             _logger.warning(
-                "No flats found in {} or unable to add them to the dataset".format(
-                    self._data_flats_url.path()
-                )
+                f"No flats found in {self._data_flats_url.path()} or unable to add them to the dataset"
             )
         else:
             image_key.extend([ImageKey.FLAT_FIELD.value] * n_flat)
@@ -545,6 +526,8 @@ class _DxFileToNxConverter(BaseConverter):
         try:
             with DatasetReader(self._data_proj_url) as proj_dataset:
                 n_proj = proj_dataset.shape[0]
+            # FIXME: avoid keeping some file open. not clear why this is needed
+            proj_dataset = None
             FrameAppender(
                 self._data_proj_url,
                 self.output_file,
@@ -554,9 +537,7 @@ class _DxFileToNxConverter(BaseConverter):
             ).process()
         except Exception:
             _logger.warning(
-                "No projections found in {} or unable to add them to the dataset".format(
-                    self._data_proj_url.path()
-                )
+                f"No projections found in {self._data_proj_url.path()} or unable to add them to the dataset"
             )
         else:
             image_key.extend([ImageKey.PROJECTION.value] * n_proj)
@@ -583,7 +564,7 @@ class _DxFileToNxConverter(BaseConverter):
             sample_grp["rotation_angle"].attrs["unit"] = "degree"
             self._path_nxtomo_attrs(root_grp)
 
-    def _read_distance(self):
+    def _read_sample_detector_distance(self):
         with EntryReader(self.input_root_url) as input_entry:
             path = "/".join(
                 ("measurement", "instrument", "sample", "setup", "detector_distance")
@@ -595,9 +576,9 @@ class _DxFileToNxConverter(BaseConverter):
                     unit = dataset.attrs["units"]
                 else:
                     unit = dataset.attrs["unit"]
-                unit = metricsystem.MetricSystem.from_value(unit).value
+                unit = _ureg(unit)
             except Exception:
-                unit = 1
+                unit = _ureg.meter
             return distance * unit
 
     def _read_energy(self):
@@ -612,14 +593,13 @@ class _DxFileToNxConverter(BaseConverter):
                     unit = dataset.attrs["unit"]
                 # patch until next tomoscan is released
                 if unit.lower() in ("mev", "megaelectronvolt"):
-                    unit = metricsystem.EnergySI.ELECTRONVOLT.value * 1e6
+                    unit = _ureg.keV * 1e3
                 elif unit.lower() in ("gev", "gigaelectronvolt"):
-                    unit = metricsystem.EnergySI.ELECTRONVOLT.value * 1e9
+                    unit = _ureg.keV * 1e6
                 else:
-                    unit = metricsystem.EnergySI.from_value(unit).value
-            except Exception as e:
-                raise e
-                unit = 1
+                    unit = _ureg(unit)
+            except Exception:
+                unit = _ureg.keV
             return energy * unit
 
     def _read_x_pixel_size(self):
@@ -637,9 +617,9 @@ class _DxFileToNxConverter(BaseConverter):
                 # patch until next tomoscan is released
                 if unit == "microns":
                     unit = "um"
-                unit = metricsystem.MetricSystem.from_value(unit).value
+                unit = _ureg(unit)
             except Exception:
-                unit = 1
+                unit = _ureg.meter
             return pixel_size * unit
 
     def _read_y_pixel_size(self):
@@ -657,7 +637,7 @@ class _DxFileToNxConverter(BaseConverter):
                 # patch until next tomoscan is released
                 if unit == "microns":
                     unit = "um"
-                unit = metricsystem.MetricSystem.from_value(unit).value
+                unit = _ureg(unit)
             except Exception:
-                unit = 1
+                unit = _ureg.meter
             return pixel_size * unit
