@@ -1,69 +1,43 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2015-2020 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
 
 """
 Base class for tomography acquisition (defined by Bliss)
 """
 
-__authors__ = [
-    "H. Payno",
-]
-__license__ = "MIT"
-__date__ = "27/11/2020"
+from __future__ import annotations
 
 import os
+import pint
+
 import h5py
-from nxtomomill.nexus.nxtomo import NXtomo
+
+from nxtomo.application.nxtomo import NXtomo
+from nxtomomill.utils.h5pyutils import from_data_url_to_virtual_source
 from nxtomomill.utils.hdf5 import EntryReader
 from nxtomomill.utils.utils import embed_url, get_file_name  # noqa F401
-from nxtomomill.utils.h5pyutils import from_data_url_to_virtual_source
 
 try:
     import hdf5plugin  # noqa F401
 except ImportError:
     pass
-from nxtomomill.plugins import (
-    get_plugins_positioners_resources,
-    get_plugins_instrument_resources,
-)
-from silx.io.url import DataUrl
-from nxtomomill.io.config import TomoHDF5Config
-from silx.io.utils import h5py_read_dataset
-from tomoscan.unitsystem import metricsystem
-from collections import OrderedDict
-import numpy
-import typing
 import logging
+from collections import OrderedDict
 
+import numpy
+from silx.io.url import DataUrl
+from silx.io.utils import h5py_read_dataset
 
+from nxtomomill.io.config import TomoHDF5Config
+
+_ureg = pint.get_application_registry()
 _logger = logging.getLogger(__name__)
 
+__all__ = ["BaseAcquisition", "get_dataset_name_from_motor"]
+
 
 def _ask_for_file_removal(file_path):
-    res = input(f"Overwrite {file_path} ? (Y/n)")
-    return res == "Y"
+    res = input(f"Overwrite {file_path} ? (y/N)").lower()
+    return res == "y"
 
 
 class BaseAcquisition:
@@ -72,8 +46,6 @@ class BaseAcquisition:
     Nexus / NXtomo compliant
     """
 
-    _SCAN_NUMBER_PATH = "measurement/scan_numbers"
-
     _ENERGY_PATH = "technique/scan/energy"
 
     _DATASET_NAME_PATH = ("technique/scan/name",)
@@ -82,9 +54,15 @@ class BaseAcquisition:
 
     _SAMPLE_NAME_PATH = ("sample/name",)
 
-    _TITLE_PATH = ("title",)
+    TITLE_PATHS = (
+        "technique/scan/sequence",
+        "title",
+    )
 
-    _INSTRUMENT_NAME_PATH = ("instrument/title",)
+    _INSTRUMENT_NAME_PATH = (
+        "technique/saving/beamline",
+        "instrument/title",
+    )
 
     _FOV_PATH = "technique/scan/field_of_view"
 
@@ -92,9 +70,17 @@ class BaseAcquisition:
 
     _NB_TOMO_PATH = ("technique/scan/nb_tomo", "technique/proj/nb_tomo")
 
-    _NB_TURNS_PATH = ("technique/proj/nb_turns",)
+    _NB_TURNS_PATH = (
+        "technique/proj/nb_turns",
+        "technique/scan/nb_turns",
+    )
 
-    _TOMO_N_PATH = "technique/proj/tomo_n", "technique/scan/tomo_n"
+    _TOMO_N_PATH = (
+        "technique/proj/tomo_n",
+        "technique/scan/tomo_n",
+        "technique/proj/proj_n",
+        "technique/scan/proj_n",
+    )
 
     _START_TIME_PATH = ("start_time",)
 
@@ -102,6 +88,8 @@ class BaseAcquisition:
 
     _TECHNIQUE_MOTOR_PATHS = ("technique/scan/motor", "technique/proj/motor")
 
+    _DETECTOR_ROI = ("technique/detector/roi",)
+
     _SOURCE_NAME = ("instrument/machine/name",)
 
     _SOURCE_TYPE = ("instrument/machine/type",)
@@ -111,9 +99,14 @@ class BaseAcquisition:
         "technique/detector/flipping",
     )
 
+    _PROPAGATION_DISTANCE_PATHS = (
+        "technique/scan/effective_propagation_distance",
+        "technique/scan/propagation_distance",
+    )
+
     def __init__(
         self,
-        root_url: typing.Union[DataUrl, None],
+        root_url: DataUrl | None,
         configuration: TomoHDF5Config,
         detector_sel_callback,
         start_index: int,
@@ -128,12 +121,15 @@ class BaseAcquisition:
         # this value is different from `.name`. Don't know if this is a bug ?
         """user can have defined already some parameter values as energy.
         The idea is to avoid asking him if """
-        self._plugins = []
-        self._plugins_pos_resources = {}
-        self._plugins_instr_resources = {}
         self._configuration = configuration
         self._start_index = start_index
 
+        # set of properties when an acquisition needs to copy dark / flat (used in the case of z-series version 3)
+        self._dark_at_start = None
+        self._dark_at_end = None
+        self._flat_at_start = None
+        self._flat_at_end = None
+
     @property
     def configuration(self):
         return self._configuration
@@ -147,11 +143,18 @@ class BaseAcquisition:
         shift_entry: int,
         input_file_path: str,
         request_input: bool,
-        plugins,
-        divide_into_sub_files,
+        divide_into_sub_files: bool,
         input_callback=None,
     ) -> tuple:
-        self.set_plugins(plugins)
+        """
+        This function will dump the acquisition to disk as an NXtomo
+
+        :param shift_entry: index of the entry to start saving new nxtomos.
+        :param input_file_path: output file path
+        :param request_input: if True the conversion can ask user some missing metadata
+        :param divide_into_sub_files: if True then create one file per NXtomo
+        :param input_callback: function to call for users to provide missing metadata
+        """
         nx_tomos = self.to_NXtomos(
             request_input=request_input,
             input_callback=input_callback,
@@ -190,9 +193,7 @@ class BaseAcquisition:
                         _logger.info("remove " + en_output_file)
                         os.remove(en_output_file)
                     elif _ask_for_file_removal(en_output_file) is False:
-                        raise OSError(
-                            "unable to overwrite {}, exit".format(en_output_file)
-                        )
+                        raise OSError(f"unable to overwrite {en_output_file}, exit")
                     else:
                         os.remove(en_output_file)
             return en_output_file, entry
@@ -203,14 +204,18 @@ class BaseAcquisition:
                 (shift_entry + i_nx_tomo), divide_sub_files=divide_into_sub_files
             )
             output_file = os.path.abspath(os.path.relpath(output_file, os.getcwd()))
-            output_file = os.path.realpath(output_file)
+            output_file = os.path.abspath(output_file)
 
-            # For pcotomo for example the data path is modified in order to handle with the splitting
+            # For multi-tomo for example the data path is modified in order to handle with the splitting
             # that does not fit at 100 % with the current API. for now there is
             # no convenient way to handle this in a better way
             assert isinstance(nx_tomo, NXtomo)
-            self._update_nxtomo_from_plugins(nx_tomo=nx_tomo)
             # embed data if requested
+            if len(nx_tomo.instrument.detector.data) == 0:
+                _logger.warning(
+                    f"No frame found for NXtomo number {i_nx_tomo}. Won't write any output for it"
+                )
+                continue
             vs_0 = nx_tomo.instrument.detector.data[0]
             if nx_tomo.detector_data_is_defined_by_url():
                 new_urls = []
@@ -240,11 +245,17 @@ class BaseAcquisition:
                     else:
                         new_vs.append(vs)
                 nx_tomo.instrument.detector.data = new_vs
+
+            # provide some extra data like origin of the input_file
+            if input_file_path is not None:
+                nx_tomo.bliss_original_files = (os.path.abspath(input_file_path),)
             # save data
             nx_tomo.save(file_path=output_file, data_path=data_path)
             # check rotation angle
             if nx_tomo.sample.rotation_angle is not None:
-                unique_angles = numpy.unique(nx_tomo.sample.rotation_angle)
+                unique_angles = numpy.unique(
+                    nx_tomo.sample.rotation_angle.to(_ureg.degree).magnitude
+                )
                 if len(unique_angles) == 1:
                     _logger.warning(
                         f"NXtomo {data_path}@{output_file} seems to have a single value ({unique_angles}) for rotation angle. Seems it fails to find correct path to the rotation angle dataset"
@@ -253,17 +264,6 @@ class BaseAcquisition:
             result.append((output_file, data_path))
         return tuple(result)
 
-    def _update_nxtomo_from_plugins(self, nx_tomo: NXtomo):
-        if len(self._plugins) > 0:
-            _logger.warning(
-                f"{len(self._plugins)} plugin found. Might modify NXtomo from them"
-            )
-        for plugin in self._plugins:
-            _logger.info(f"Apply plugin {plugin.name}")
-            plugin.update_nx_tomo(
-                nx_tomo=nx_tomo,
-            )
-
     def to_NXtomos(self, request_input, input_callback, check_tomo_n=True) -> tuple:
         raise NotImplementedError("Base class")
 
@@ -275,27 +275,6 @@ class BaseAcquisition:
         """
         return self.configuration.raises_error
 
-    @property
-    def is_xrd_ct(self):
-        """Is this an XRD-CT acquisition"""
-        raise NotImplementedError("Base class")
-
-    @property
-    def require_x_translation(self):
-        """is `x_translation` expected"""
-        raise NotImplementedError("Base class")
-
-    @property
-    def require_z_translation(self):
-        """is `z_translation` expected"""
-        raise NotImplementedError("Base class")
-
-    @property
-    def has_diode(self):
-        """is the acquisition expect to have a diode (instead of an energy
-        field)"""
-        raise NotImplementedError("Base class")
-
     @property
     def root_url(self):
         return self._root_url
@@ -313,19 +292,49 @@ class BaseAcquisition:
     def is_different_sequence(self, entry):
         """
         Can we have several entries 1.1, 1.2, 1.3... to consider.
-        This is the case for XRD-CT where 1.1, 1.2, 1.3 should be consider as
-        being part of the same sequence. Not for 'standard tomography'
         """
         raise ValueError("Base class")
 
+    def is_part_of_same_series(self, other: BaseAcquisition) -> bool:
+        if not isinstance(other, BaseAcquisition):
+            raise TypeError("Can only compar two z-series")
+        if self.root_url is None or other.root_url is None:
+            return False
+
+        def read_sample_node(url: DataUrl) -> str | None:
+            with EntryReader(url=url) as entry:
+                sample_node = self._get_sample_node(entry)
+                if sample_node is None:
+                    return None
+                sample_name = sample_node.get("name", None)
+                if sample_name is not None:
+                    return h5py_read_dataset(sample_name)
+                return None
+
+        self_sample_name = read_sample_node(self.root_url)
+        other_sample_name = read_sample_node(other.root_url)
+        if self_sample_name is None or other_sample_name is None:
+            return False
+
+        def filter_scan_index(sample_name: str) -> str:
+            # for z_series the sample name is post pone with _{sample_name} that we need to filter
+            parts = sample_name.split("_")
+            if len(parts) > 1 and numpy.char.isnumeric(parts[-1]):
+                return "_".join(parts[:-1])
+            return sample_name
+
+        return filter_scan_index(self_sample_name) == filter_scan_index(
+            other_sample_name
+        )
+
     @staticmethod
     def _get_node_values_for_frame_array(
         node: h5py.Group,
-        n_frame: int,
-        keys: typing.Iterable,
+        n_frame: int | None,
+        keys: list | tuple,
         info_retrieve,
         expected_unit,
-    ):
+    ) -> tuple[numpy.ndarray | None, str]:
         def get_values():
             # this is a two step process: first step we parse all the
             # the keys until we found one with the expected length
@@ -357,9 +366,7 @@ class BaseAcquisition:
         values, unit = get_values()
         if values is None:
             raise ValueError(
-                "Unable to retrieve {info} for {node_name}. Was looking for "
-                "{keys} datasets"
-                "".format(info=info_retrieve, node_name=node.name, keys=keys)
+                f"Unable to retrieve {info_retrieve} for {node.name}. Was looking for {keys} datasets"
             )
         elif n_frame is None:
             return values, unit
@@ -369,7 +376,7 @@ class BaseAcquisition:
             return values.tolist(), unit
         elif len(values) == (n_frame + 1):
             # for now we can have one extra position for rotation,
-            # x_translation...
+            # translation_x...
             # because saved after the last projection. It is recording the
             # motor position. For example in this case: 1 is the motor movement
             # (saved) and 2 is the acquisition
@@ -379,6 +386,18 @@ class BaseAcquisition:
             # -----     -----     -----
             #
             return values[:-1].tolist(), unit
+        elif len(values) > n_frame:
+            _logger.warning(
+                f"Incoherent number of values found for {info_retrieve}. Can come from an acquisition canceled. Else please investigate."
+            )
+            # in this case only get the values which have a frame
+            return values[0:n_frame], unit
+        elif len(values) < n_frame:
+            _logger.warning(
+                f"Incoherent number of values found for {info_retrieve}. Can come from an acquisition canceled. Else please investigate."
+            )
+            # in this case append 0 to existing values. Maybe -1 would be better ?
+            return list(values) + [0] * (n_frame - values), unit
         elif len(values) == 1:
             return numpy.array([values[0]] * n_frame), unit
         else:
@@ -393,7 +412,7 @@ class BaseAcquisition:
         raise NotImplementedError("Base class")
 
     @staticmethod
-    def _get_unit(node: h5py.Dataset, default_unit):
+    def _get_unit(node: h5py.Dataset, default_unit) -> str:
         """Simple process to retrieve unit from an attribute"""
         if "unit" in node.attrs:
             return node.attrs["unit"]
@@ -412,7 +431,7 @@ class BaseAcquisition:
         return entry_node["instrument"]
 
     @staticmethod
-    def _get_positioners_node(entry_node):
+    def _get_positioners_node(entry_node: h5py.Group) -> h5py.Group | None:
         if not isinstance(entry_node, h5py.Group):
             raise TypeError("entry_node is expected to be a h5py.Group")
         parent_node = BaseAcquisition._get_instrument_node(entry_node)
@@ -422,7 +441,7 @@ class BaseAcquisition:
             return None
 
     @staticmethod
-    def _get_measurement_node(entry_node):
+    def _get_measurement_node(entry_node: h5py.Group) -> h5py.Group | None:
         if not isinstance(entry_node, h5py.Group):
             raise TypeError("entry_node is expected to be a h5py.Group")
         if "measurement" in entry_node:
@@ -431,7 +450,7 @@ class BaseAcquisition:
             return None
 
     @staticmethod
-    def _get_machine_node(entry_node):
+    def _get_machine_node(entry_node: h5py.Group) -> h5py.Group | None:
         if not isinstance(entry_node, h5py.Group):
             raise TypeError("entry_node is expected to be a h5py.Group")
         if "instrument/machine" in entry_node:
@@ -439,11 +458,28 @@ class BaseAcquisition:
         else:
             return None
 
-    def _read_rotation_motor_name(self) -> typing.Union[str, None]:
+    @staticmethod
+    def _get_sample_node(entry_node: h5py.Group) -> h5py.Group | None:
+        if not isinstance(entry_node, h5py.Group):
+            raise TypeError("entry_node is expected to be a h5py.Group")
+        if "sample" in entry_node:
+            return entry_node["sample"]
+        else:
+            return None
+
+    @staticmethod
+    def _get_scan_flags(entry_node: h5py.Group) -> h5py.Group | None:
+        if not isinstance(entry_node, h5py.Group):
+            raise TypeError("entry_node is expected to be a h5py.Group")
+        if "technique/scan_flags" in entry_node:
+            return entry_node["technique/scan_flags"]
+        else:
+            return None
+
+    def _read_rotation_motor_name(self) -> str | None:
         """read rotation motor from root_url/technique/scan/motor
 
         :return: name of the motor used for rotation. None if cannot find
-        :rtype: Union[tuple, None]
         """
         if self._root_url is None:
             _logger.warning("no root url. Unable to read rotation motor")
@@ -465,17 +501,15 @@ class BaseAcquisition:
                             return rotation_motor
                     else:
                         _logger.warning(
-                            "{} unable to find rotation motor from {}".format(
-                                motor_path, self._root_url
-                            )
+                            f"{motor_path} unable to find rotation motor from {self._root_url}"
                         )
                 return None
 
-    def _get_electric_current(self, root_node) -> list:
+    def _get_machine_current(self, root_node) -> None | pint.Quantity:
         """retrieve electric current provide a time stamp for each of them"""
         if root_node is None:
             _logger.warning("no root url. Unable to read electric current")
-            return None, None
+            return None
         else:
             grps = [
                 root_node,
@@ -490,36 +524,45 @@ class BaseAcquisition:
 
             for grp in grps:
                 try:
-                    elec_current, unit = self._get_node_values_for_frame_array(
+                    mach_current, unit = self._get_node_values_for_frame_array(
                         node=grp,
-                        keys=self.configuration.machine_electric_current_keys,
-                        info_retrieve="machine electric current",
+                        keys=self.configuration.machine_current_keys,
+                        info_retrieve="machine current",
                         expected_unit="mA",
                         n_frame=None,
                     )
                 except (ValueError, KeyError):
                     pass
                 else:
-                    # handle case where elec_current is a scalar. Cast it to list before return
-                    if numpy.isscalar(elec_current):
-                        elec_current = [
-                            elec_current,
+                    # handle case where mach_current is a scalar. Cast it to list before return
+                    if numpy.isscalar(mach_current):
+                        mach_current = [
+                            mach_current,
                         ]
 
-                    return elec_current, unit
+                    return mach_current * _ureg(unit)
             else:
                 _logger.warning(
-                    f"Unable to retrieve machine electric current for {root_node.name}"
+                    f"Unable to retrieve machine current for {root_node.name}"
                 )
 
-            return None, None
+            return None
 
-    def _get_rotation_angle(self, root_node, n_frame) -> tuple:
-        """return the list of rotation angle for each frame"""
+    def _get_rotation_angle(self, root_node: h5py.Group, n_frame: int) -> pint.Quantity:
+        """
+        return the list of rotation angle for each frame.
+        If unable to find it will either:
+        * raise an error (if 'raise_error_if_issue' is True)
+        * return a zeros for all required angles (kwnow from n_frame)
+        """
         if not isinstance(root_node, h5py.Group):
             raise TypeError("root_node is expected to be a h5py.Group")
 
-        for grp in self._get_positioners_node(root_node), root_node:
+        for grp in (
+            self._get_positioners_node(root_node),
+            root_node,
+            self._get_measurement_node(root_node),
+        ):
             try:
                 angles, unit = self._get_node_values_for_frame_array(
                     node=grp,
@@ -531,7 +574,7 @@ class BaseAcquisition:
             except (ValueError, KeyError):
                 pass
             else:
-                return angles, unit
+                return angles * _ureg(unit)
 
         mess = f"Unable to find rotation angle for {root_node.name}"
         if self.raise_error_if_issue:
@@ -539,103 +582,123 @@ class BaseAcquisition:
         else:
             mess += "default value will be set. (0)"
             _logger.warning(mess)
-            return [0] * n_frame, "degree"
+            return ([0] * n_frame) * _ureg.degree
 
-    def _get_x_translation(self, root_node, n_frame) -> tuple:
+    def _get_sample_x(self, root_node, n_frame) -> pint.Quantity:
         """return the list of translation for each frame"""
         for grp in self._get_positioners_node(root_node), root_node:
             try:
                 x_tr, unit = self._get_node_values_for_frame_array(
                     node=grp,
                     n_frame=n_frame,
-                    keys=self.configuration.x_trans_keys,
-                    info_retrieve="x translation",
+                    keys=self.configuration.sample_x_keys,
+                    info_retrieve="sample x translation",
                     expected_unit="mm",
                 )
-                x_tr = (
-                    numpy.asarray(x_tr)
-                    * metricsystem.MetricSystem.from_value(unit).value
-                )
+                x_tr = numpy.asarray(x_tr)
             except (ValueError, KeyError):
                 pass
             else:
-                return x_tr, "m"
+                return x_tr * pint.Unit(unit)
 
-        mess = f"Unable to find x translation for {self.root_url.path()}"
+        mess = f"Unable to find sample x for {root_node.name}"
         if self.raise_error_if_issue:
             raise ValueError(mess)
         else:
             mess += "default value will be set. (0)"
             _logger.warning(mess)
-            return [0] * n_frame, "m"
+            return ([0] * n_frame) * _ureg.meter
 
-    def _get_y_translation(self, root_node, n_frame) -> tuple:
+    def _get_sample_y(self, root_node, n_frame) -> pint.Quantity:
         """return the list of translation for each frame"""
         for grp in self._get_positioners_node(root_node), root_node:
             try:
                 y_tr, unit = self._get_node_values_for_frame_array(
                     node=grp,
                     n_frame=n_frame,
-                    keys=self.configuration.y_trans_keys,
-                    info_retrieve="y translation",
+                    keys=self.configuration.sample_y_keys,
+                    info_retrieve="sample y translation",
                     expected_unit="mm",
                 )
-                y_tr = (
-                    numpy.asarray(y_tr)
-                    * metricsystem.MetricSystem.from_value(unit).value
-                )
+                y_tr = numpy.asarray(y_tr)
             except (ValueError, KeyError):
                 pass
             else:
-                return y_tr, "m"
+                return y_tr * pint.Unit(unit)
 
-        mess = f"Unable to find y translation for {self.root_url.path()}"
+        mess = f"Unable to find sample y for {root_node.name}"
         if self.raise_error_if_issue:
             raise ValueError(mess)
         else:
             mess += "default value will be set. (0)"
             _logger.warning(mess)
-            return [0] * n_frame, "m"
+            return ([0] * n_frame) * _ureg.meter
+
+    def _get_translation_y(self, root_node, n_frame) -> pint.quantity:
+        """return the list of translation for each frame"""
+        for grp in self._get_positioners_node(root_node), root_node:
+            try:
+                y_tr, unit = self._get_node_values_for_frame_array(
+                    node=grp,
+                    n_frame=n_frame,
+                    keys=self.configuration.translation_y_keys,
+                    info_retrieve="y base translation",
+                    expected_unit="mm",
+                )
+                y_tr = numpy.asarray(y_tr)
+            except (ValueError, KeyError):
+                pass
+            else:
+                return y_tr * pint.Unit(unit)
+
+        mess = f"Unable to find translation y for {root_node.name}"
+        if self.raise_error_if_issue:
+            raise ValueError(mess)
+        else:
+            mess += "default value will be set. (0)"
+            _logger.warning(mess)
+            return ([0] * n_frame) * _ureg.meter
 
     @staticmethod
-    def get_z_translation_frm(root_node, n_frame: int, configuration: TomoHDF5Config):
+    def get_translation_z_frm(
+        root_node, n_frame: int, configuration: TomoHDF5Config
+    ) -> pint.Quantity:
         for grp in BaseAcquisition._get_positioners_node(root_node), root_node:
             try:
                 z_tr, unit = BaseAcquisition._get_node_values_for_frame_array(
                     node=grp,
                     n_frame=n_frame,
-                    keys=configuration.z_trans_keys,
+                    keys=configuration.translation_z_keys,
                     info_retrieve="z translation",
                     expected_unit="mm",
                 )
-                z_tr = (
-                    numpy.asarray(z_tr)
-                    * metricsystem.MetricSystem.from_value(unit).value
-                )
+                z_tr = numpy.asarray(z_tr)
             except (ValueError, KeyError):
                 pass
             else:
-                return z_tr, "m"
+                return z_tr * _ureg(unit)
 
-        mess = f"Unable to find z translation on node {root_node.name}"
+        mess = f"Unable to find translation z on node {root_node.name}"
         if configuration.raises_error:
             raise ValueError(mess)
         else:
             mess += "default value will be set. (0)"
             _logger.warning(mess)
-            return [0] * n_frame, "m"
+            return ([0] * n_frame) * _ureg.meter
 
-    def _get_z_translation(self, root_node, n_frame) -> tuple:
-        """return the list of translation for each frame"""
-        return self.get_z_translation_frm(
+    def _get_translation_z(self, root_node, n_frame) -> pint.Quantity:
+        """return the list of translation z for each frame"""
+        return self.get_translation_z_frm(
             root_node=root_node,
             n_frame=n_frame,
             configuration=self.configuration,
         )
 
-    def _get_expo_time(self, root_node, n_frame, detector_node) -> tuple:
+    def _get_expo_time(self, root_node, n_frame, detector_node) -> pint.Quantity:
         """return expo time for each frame"""
-        for grp in detector_node["acq_parameters"], root_node:
+        for grp in (detector_node.get("acq_parameters", None), root_node):
+            if grp is None:
+                continue
             try:
                 expo, unit = self._get_node_values_for_frame_array(
                     node=grp,
@@ -647,7 +710,7 @@ class BaseAcquisition:
             except (ValueError, KeyError):
                 pass
             else:
-                return expo, unit
+                return expo * pint.Unit(unit)
 
         mess = f"Unable to find frame exposure time on entry {self.root_url.path()}"
         if self.raise_error_if_issue:
@@ -655,64 +718,7 @@ class BaseAcquisition:
         else:
             mess += "default value will be set. (0)"
             _logger.warning(mess)
-            return 0, "s"
-
-    def __get_plugin_xxx_resource(self, resource_name: str, n_frame, nodes):
-        if not isinstance(resource_name, str):
-            raise TypeError("resource_name is expected to be a str")
-        for grp in nodes:
-            try:
-                values, _ = self._get_node_values_for_frame_array(
-                    node=grp,
-                    n_frame=n_frame,
-                    keys=(resource_name,),
-                    info_retrieve=resource_name,
-                    expected_unit=None,
-                )
-            except (ValueError, KeyError):
-                pass
-            else:
-                return values, None
-        mess = f"Unable to find {resource_name} on entry {self.root_url.path()}"
-        if self.raise_error_if_issue:
-            raise ValueError(mess)
-        else:
-            _logger.warning(mess)
-            return None, None
-
-    def _get_plugin_pos_resource(self, root_node, resource_name, n_frame):
-        """Reach a path provided by a plugin. In this case units are not
-        managed"""
-        return self.__get_plugin_xxx_resource(
-            resource_name=resource_name,
-            n_frame=n_frame,
-            nodes=(self._get_positioners_node(root_node), root_node),
-        )
-
-    def _get_plugin_instr_resource(self, root_node, resource_name, n_frame):
-        """Reach a path provided by a plugin. In this case units are not
-        managed"""
-        return self.__get_plugin_xxx_resource(
-            resource_name=resource_name,
-            n_frame=n_frame,
-            nodes=(self._get_instrument_node(root_node), root_node),
-        )
-
-    def set_plugins(self, plugins):
-        """
-
-        :param list plugins: list of plugins to call
-        """
-        self._plugins = plugins
-        _plugins_req_resources = get_plugins_positioners_resources(plugins)
-        self._plugins_pos_resources = {}
-        for requested_resource in _plugins_req_resources:
-            self._plugins_pos_resources[requested_resource] = []
-
-        _plugins_req_resources = get_plugins_instrument_resources(plugins)
-        self._plugins_instr_resources = {}
-        for requested_resource in _plugins_req_resources:
-            self._plugins_instr_resources[requested_resource] = []
+            return 0 * _ureg.second
 
     def get_axis_scale_types(self):
         """
@@ -726,6 +732,26 @@ class BaseAcquisition:
         else:
             return self.root_url.path()
 
+    def get_detector_roi(self):
+        if self._root_url is None:
+            _logger.warning("no root url. Unable to read detector roi")
+            return None
+        else:
+            with EntryReader(self._root_url) as entry:
+                for roi_path in self._DETECTOR_ROI:
+                    if roi_path in entry:
+                        try:
+                            roi = h5py_read_dataset(numpy.asarray(entry[roi_path]))
+                        except Exception as e:
+                            _logger.error(e)
+                        else:
+                            return roi
+                    else:
+                        _logger.warning(
+                            f"{roi_path} unable to find detector roi from {self._root_url}"
+                        )
+                return None
+
 
 def get_dataset_name_from_motor(motors, motor_name):
     motors = numpy.asarray(motors)
