@@ -1,42 +1,71 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2015-2020 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-#
-# ###########################################################################
-
-
-__authors__ = [
-    "H. Payno",
-]
-__license__ = "MIT"
-__date__ = "21/04/2022"
-
 
+from __future__ import annotations
 import configparser
-from typing import Union, Iterable
-from nxtomomill.utils import FileExtension
-from nxtomomill.nexus.nxdetector import FieldOfView
 import logging
+from typing import Iterable
+
+
+from nxtomomill.io.config.optionlevel import OptionLevel, filter_options_level_items
+from nxtomo.nxobject.nxdetector import FieldOfView
+from nxtomomill.utils import FileExtension
+from nxtomo.nxobject.nxsource import ProbeType, SourceType
+
+__all__ = ["ConfigBase", "ConfigSourceSection"]
+
+
+class ConfigSourceSection:
+    # SOURCE SECTION
+
+    SOURCE_SECTION_DK = "SOURCE_SECTION"
+
+    INSTRUMENT_NAME_DK = "instrument_name"
+
+    SOURCE_NAME_DK = "source_name"
+
+    SOURCE_TYPE_DK = "source_type"
+
+    SOURCE_PROBE_DK = "source_probe"
+
+    COMMENTS_SOURCE_SECTION_DK = {
+        SOURCE_SECTION_DK: "section dedicated to source definition.\n",
+        INSTRUMENT_NAME_DK: "name of the instrument",
+        SOURCE_NAME_DK: "name of the source",
+        SOURCE_TYPE_DK: f"type of the source. Must be one of {[item.value for item in SourceType]}",
+        SOURCE_PROBE_DK: f"source probe. Must be one of {[item.value for item in ProbeType]}",
+    }
+
+    LEVEL_SOURCE_SECTION = {
+        INSTRUMENT_NAME_DK: OptionLevel.ADVANCED,
+        SOURCE_NAME_DK: OptionLevel.ADVANCED,
+        SOURCE_TYPE_DK: OptionLevel.ADVANCED,
+        SOURCE_PROBE_DK: OptionLevel.ADVANCED,
+    }
+
+    def load_source_section(self, dict_: dict) -> None:
+        if self.INSTRUMENT_NAME_DK in dict_:
+            self.instrument_name = dict_.get(self.INSTRUMENT_NAME_DK)
+        if self.SOURCE_NAME_DK in dict_:
+            self.source_name = dict_.get(self.SOURCE_NAME_DK)
+        if self.SOURCE_TYPE_DK in dict_:
+            self.source_type = dict_[self.SOURCE_TYPE_DK]
+        if self.SOURCE_PROBE_DK in dict_:
+            self.source_probe = dict_[self.SOURCE_PROBE_DK]
+
+    def _source_section_to_dict(self, level) -> dict:
+        res = {
+            self.INSTRUMENT_NAME_DK: self.instrument_name or "",
+            self.SOURCE_NAME_DK: self.source_name or "",
+            self.SOURCE_TYPE_DK: (
+                self.source_type.value if self.source_type is not None else ""
+            ),
+            self.SOURCE_PROBE_DK: (
+                self.source_probe.value if self.source_probe is not None else ""
+            ),
+        }
+        return filter_options_level_items(
+            dict_=res, level=level, level_ref=self.LEVEL_SOURCE_SECTION
+        )
 
 
 class ConfigBase:
@@ -50,7 +79,7 @@ class ConfigBase:
         self._file_extension = FileExtension.NX
         self._log_level = logging.WARNING
         self._field_of_view = None
-        self._machine_electric_current_keys = None
+        self._machine_current_keys = None
 
     def __setattr__(self, __name, __value):
         if self.__isfrozen and not hasattr(self, __name):
@@ -59,11 +88,11 @@ class ConfigBase:
             super().__setattr__(__name, __value)
 
     @property
-    def output_file(self) -> Union[None, str]:
+    def output_file(self) -> str | None:
         return self._output_file
 
     @output_file.setter
-    def output_file(self, output_file: Union[None, str]):
+    def output_file(self, output_file: str | None):
         if not isinstance(output_file, (str, type(None))):
             raise TypeError("'input_file' should be None or an instance of Iterable")
         elif output_file == "":
@@ -88,7 +117,7 @@ class ConfigBase:
 
     @file_extension.setter
     def file_extension(self, file_extension: str):
-        self._file_extension = FileExtension.from_value(file_extension)
+        self._file_extension = FileExtension(file_extension)
 
     @property
     def log_level(self):
@@ -102,21 +131,20 @@ class ConfigBase:
         self.__isfrozen = freeze
 
     @property
-    def field_of_view(self) -> Union[None, FieldOfView]:
+    def field_of_view(self) -> FieldOfView | None:
         return self._field_of_view
 
     @field_of_view.setter
-    def field_of_view(self, fov: Union[None, FieldOfView, str]):
+    def field_of_view(self, fov: FieldOfView | str | None):
         if fov is None:
             self._field_of_view = fov
         elif isinstance(fov, str):
-            self._field_of_view = FieldOfView.from_value(fov.title())
+            self._field_of_view = FieldOfView(fov.title())
         elif isinstance(fov, FieldOfView):
             self._field_of_view = fov
         else:
             raise TypeError(
-                "fov is expected to be None, a string or "
-                "FieldOfView. Not {}".format(type(fov))
+                f"fov is expected to be None, a string or FieldOfView. Not {type(fov)}"
             )
 
     @property
@@ -134,54 +162,26 @@ class ConfigBase:
             self._rot_angle_keys = keys
 
     @property
-    def x_trans_keys(self) -> Iterable:
-        return self._x_trans_keys
+    def translation_z_keys(self) -> Iterable:
+        return self._translation_z_keys
 
-    @x_trans_keys.setter
-    def x_trans_keys(self, keys) -> None:
+    @translation_z_keys.setter
+    def translation_z_keys(self, keys) -> None:
         if not isinstance(keys, Iterable) or isinstance(keys, str):
             raise TypeError("'keys' should be an Iterable")
         else:
             for elmt in keys:
                 if not isinstance(elmt, str):
                     raise TypeError("keys elmts are expected to be str")
-            self._x_trans_keys = keys
+            self._translation_z_keys = keys
 
     @property
-    def y_trans_keys(self) -> Iterable:
-        return self._y_trans_keys
+    def machine_current_keys(self) -> Iterable:
+        return self._machine_current_keys
 
-    @y_trans_keys.setter
-    def y_trans_keys(self, keys) -> None:
-        if not isinstance(keys, Iterable) or isinstance(keys, str):
-            raise TypeError("'keys' should be an Iterable")
-        else:
-            for elmt in keys:
-                if not isinstance(elmt, str):
-                    raise TypeError("keys elmts are expected to be str")
-            self._y_trans_keys = keys
-
-    @property
-    def z_trans_keys(self) -> Iterable:
-        return self._z_trans_keys
-
-    @z_trans_keys.setter
-    def z_trans_keys(self, keys) -> None:
-        if not isinstance(keys, Iterable) or isinstance(keys, str):
-            raise TypeError("'keys' should be an Iterable")
-        else:
-            for elmt in keys:
-                if not isinstance(elmt, str):
-                    raise TypeError("keys elmts are expected to be str")
-            self._z_trans_keys = keys
-
-    @property
-    def machine_electric_current_keys(self) -> Iterable:
-        return self._machine_electric_current_keys
-
-    @machine_electric_current_keys.setter
-    def machine_electric_current_keys(self, keys: Iterable) -> None:
-        self._machine_electric_current_keys = keys
+    @machine_current_keys.setter
+    def machine_current_keys(self, keys: Iterable) -> None:
+        self._machine_current_keys = keys
 
     def to_dict(self) -> dict:
         """convert the configuration to a dictionary"""
