@@ -1,48 +1,30 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2015-2020 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
 
 """
 contains the HDF5ConfigHandler
 """
 
-__authors__ = ["H. Payno"]
-__license__ = "MIT"
-__date__ = "08/03/2021"
+from __future__ import annotations
 
-
-from .hdf5config import TomoHDF5Config
-from .hdf5config import XRD3DHDF5Config
-from nxtomomill import utils
-from nxtomomill.utils import Format
-from nxtomomill.io.utils import filter_str_def
-from typing import Union
 import logging
 import os
 
+from nxtomomill import utils
+from nxtomomill.io.utils import filter_str_def
+from nxtomomill.converter.hdf5.utils import get_default_output_file
+
+from .hdf5config import TomoHDF5Config
+
 _logger = logging.getLogger(__name__)
 
+__all__ = [
+    "SETTABLE_PARAMETERS_UNITS",
+    "SETTABLE_PARAMETERS_TYPE",
+    "SETTABLE_PARAMETERS",
+    "BaseHDF5ConfigHandler",
+    "TomoHDF5ConfigHandler",
+]
+
 SETTABLE_PARAMETERS_UNITS = {
     "energy": "kev",
     "x_pixel_size": "m",
@@ -64,10 +46,9 @@ def _extract_param_value(key_values):
     """extract all the key / values elements from the str_list. Expected
     format is `param_1_name param_1_value param_2_name param_2_value ...`
 
-    :param str str_list: raw input string as `param_1_name param_1_value
+    :param str_list: raw input string as `param_1_name param_1_value
                          param_2_name param_2_value ...`
     :return: dict of tuple (param_name, param_value)
-    :rtype: dict
     """
     if len(key_values) % 2 != 0:
         raise ValueError(
@@ -86,7 +67,7 @@ def _extract_param_value(key_values):
     res = {}
     for name, value in pairwise(key_values):
         if name not in SETTABLE_PARAMETERS:
-            raise ValueError("parameters {} is not managed".format(name))
+            raise ValueError(f"parameters {name} is not managed")
         if name in SETTABLE_PARAMETERS_TYPE:
             type_ = SETTABLE_PARAMETERS_TYPE[name]
             if type_ is not None:
@@ -97,8 +78,11 @@ def _extract_param_value(key_values):
 
 
 class BaseHDF5ConfigHandler:
-    """Class to handle inputs to HDF5Config.
-    And insure there is no opposite Information
+    """
+    Class taking as input the CLI options and the user configuration file.
+    Raises error if there is incoherence between the two (like if the CLI provides image_keys values AND if the configuration file provides the same input).
+
+    This class will produce the instance of TomoHDF5Config to be used for the conversion
     """
 
     @staticmethod
@@ -116,22 +100,13 @@ class BaseHDF5ConfigHandler:
         else:
             return "warning"
 
-    @staticmethod
-    def conv_xrd_ct_to_format(str_bool):
-        if str_bool in ("True", True):
-            return Format.XRD_CT
-        elif str_bool in ("False", False):
-            return Format.STANDARD
-        else:
-            return None
-
     def __init__(self, argparse_options, raise_error=True):
         self._argparse_options = argparse_options
         self._config = None
         self.build_configuration(raise_error=raise_error)
 
     @property
-    def configuration(self) -> Union[None, TomoHDF5Config]:
+    def configuration(self) -> TomoHDF5Config | None:
         return self._config
 
     @property
@@ -142,11 +117,11 @@ class BaseHDF5ConfigHandler:
         raise NotImplementedError("BaseClass")
 
     def _create_HDF5_config(self):
-        raise NotImplementedError("BAse class")
+        raise NotImplementedError("Base class")
 
     def build_configuration(self, raise_error) -> bool:
         """
-        :param bool raise_error: raise error if encounter some errors. Else
+        :param raise_error: raise error if encounter some errors. Else
                                  display a log message
         :return: True if the settings are valid
         """
@@ -180,14 +155,23 @@ class BaseHDF5ConfigHandler:
                 "command line and from the configuration file"
             )
         if config.output_file is None:
-            input_file, input_file_ext = os.path.splitext(config.input_file)
             if config.file_extension is None:
-                err = "If no outputfile provided you should provide the " "extension"
-                if raise_error:
-                    raise ValueError(err)
-                else:
-                    _logger.error(err)
-            config.output_file = input_file + config.file_extension.value
+                err = "If no output file provided you should provide the extension"
+                raise ValueError(err)
+
+            config.output_file = get_default_output_file(
+                input_file=config.input_file,
+                output_file_extension=config.file_extension.value,
+            )
+        elif os.path.isdir(config.output_file):
+            # if the output file is only a directory: consider we want the same file basename with default extension
+            # in this directory
+            input_file, _ = os.path.splitext(config.input_file)
+            config.output_file = os.path.join(
+                os.path.abspath(config.output_file),
+                os.path.basename(input_file) + config.file_extension.value,
+            )
+
         # set parameter from the arg parse options
         # key is the name of the argparse option.
         # value is a tuple: (name of the setter in the HDF5Config,
@@ -202,6 +186,11 @@ class BaseHDF5ConfigHandler:
 
 
 class TomoHDF5ConfigHandler(BaseHDF5ConfigHandler):
+    """
+    Create a TomoHDF5Config from argparse option provided from the CLI.
+    In the case a configuration is provided it also check that a parameter is not defined twice (from the CLI and from the configuration file)
+    """
+
     def _create_HDF5_config(self):
         if self.argparse_options.config:
             return TomoHDF5Config.from_cfg_file(self.argparse_options.config)
@@ -217,6 +206,7 @@ class TomoHDF5ConfigHandler(BaseHDF5ConfigHandler):
             "overwrite": ("overwrite", self.conv_str_to_bool),
             "file_extension": ("file_extension", filter_str_def),
             "single_file": ("single_file", self.conv_str_to_bool),
+            "no_master_file": ("no_master_file", self.conv_str_to_bool),
             "debug": ("log_level", self.conv_log_level),
             "entries": ("entries", self.get_tuple_of_keys_from_cmd),
             "ignore_sub_entries": (
@@ -227,9 +217,12 @@ class TomoHDF5ConfigHandler(BaseHDF5ConfigHandler):
             "raises_error": ("raises_error", self.conv_str_to_bool),
             "field_of_view": ("field_of_view", filter_str_def),
             "request_input": ("request_input", self.conv_str_to_bool),
-            "x_trans_keys": ("x_trans_keys", self.get_tuple_of_keys_from_cmd),
-            "y_trans_keys": ("y_trans_keys", self.get_tuple_of_keys_from_cmd),
-            "z_trans_keys": ("z_trans_keys", self.get_tuple_of_keys_from_cmd),
+            "sample_x_keys": ("sample_x_keys", self.get_tuple_of_keys_from_cmd),
+            "sample_y_keys": ("sample_y_keys", self.get_tuple_of_keys_from_cmd),
+            "translation_z_keys": (
+                "translation_z_keys",
+                self.get_tuple_of_keys_from_cmd,
+            ),
             "rot_angle_keys": ("rotation_angle_keys", self.get_tuple_of_keys_from_cmd),
             "sample_detector_distance_paths": (
                 "sample_detector_distance_paths",
@@ -246,8 +239,12 @@ class TomoHDF5ConfigHandler(BaseHDF5ConfigHandler):
                 "zserie_init_titles",
                 self.get_tuple_of_keys_from_cmd,
             ),
-            "init_pcotomo_titles": (
-                "pcotomo_init_titles",
+            "init_multi_tomo_titles": (
+                "multitomo_init_titles",
+                self.get_tuple_of_keys_from_cmd,
+            ),
+            "init_back_and_forth_titles": (
+                "back_and_forth_init_titles",
                 self.get_tuple_of_keys_from_cmd,
             ),
             "dark_titles": ("dark_titles", self.get_tuple_of_keys_from_cmd),
@@ -284,15 +281,15 @@ class TomoHDF5ConfigHandler(BaseHDF5ConfigHandler):
                 "dark_titles",
                 "init_zserie_titles",
                 "init_titles",
-                "init_pcotomo_titles",
+                "init_multi_tomo_titles",
                 "x_pixel_size_key",
                 "y_pixel_size_key",
                 "acq_expo_time_keys",
                 "rot_angle_keys",
                 "valid_camera_names",
-                "z_trans_keys",
-                "y_trans_keys",
-                "x_trans_keys",
+                "translation_z_keys",
+                "sample_y_keys",
+                "sample_x_keys",
                 "request_input",
                 "raises_error",
                 "ignore_sub_entries",
@@ -300,6 +297,7 @@ class TomoHDF5ConfigHandler(BaseHDF5ConfigHandler):
                 "debug",
                 "overwrite",
                 "single_file",
+                "no_master_file",
                 "file_extension",
                 "field_of_view",
                 "duplicate_data",
@@ -307,49 +305,7 @@ class TomoHDF5ConfigHandler(BaseHDF5ConfigHandler):
                 if getattr(options, opt):
                     duplicated_inputs.append(opt)
             if len(duplicated_inputs) > 0:
-                err = "You provided a configuration file and inputs " "for {}".format(
-                    duplicated_inputs
-                )
-                if raise_error:
-                    raise ValueError(err)
-                else:
-                    _logger.error(err)
-                return False
-
-
-class XRD3DHDF5ConfigHandler(TomoHDF5ConfigHandler):
-    def _create_HDF5_config(self):
-        if self.argparse_options.config:
-            return XRD3DHDF5Config.from_cfg_file(self.argparse_options.config)
-        else:
-            return XRD3DHDF5Config()
-
-    def _map_option_to_config_param(self, config, options):
-        config = super()._map_option_to_config_param(config, options)
-
-        mapping = {
-            "rocking_keys": ("rocking_keys", self.get_tuple_of_keys_from_cmd),
-        }
-        for argparse_name, (config_name, format_fct) in mapping.items():
-            argparse_value = getattr(options, argparse_name)
-            if argparse_value is not None:
-                value = format_fct(argparse_value)  # pylint: disable=E1102
-                setattr(config, config_name, value)
-        return config
-
-    def _check_argparse_options(self, raise_error):
-        super()._check_argparse_options(raise_error)
-        options = self.argparse_options
-        if options.config is not None:
-            # check no other option are provided
-            duplicated_inputs = []
-            for opt in ("rocking_keys",):
-                if getattr(options, opt):
-                    duplicated_inputs.append(opt)
-            if len(duplicated_inputs) > 0:
-                err = "You provided a configuration file and inputs " "for {}".format(
-                    duplicated_inputs
-                )
+                err = f"You provided a configuration file and inputs for {duplicated_inputs}"
                 if raise_error:
                     raise ValueError(err)
                 else:
