@@ -1,64 +1,35 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2015-2020 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
 
 """
 contains the FrameGroup
 """
 
-__authors__ = [
-    "H. Payno",
-]
-__license__ = "MIT"
-__date__ = "17/03/2021"
+from __future__ import annotations
 
+import logging
+from typing import Iterable
 
 from silx.io.url import DataUrl
-from typing import Union
-from .acquisitionstep import AcquisitionStep
 from silx.utils.enum import Enum as _Enum
-from typing import Iterable
-import logging
+
+from .acquisitionstep import AcquisitionStep
 
 _logger = logging.getLogger(__name__)
 
 
+__all__ = ["FrameGroup", "filter_acqui_frame_type"]
+
+
 class FrameGroup:
     """class used to store information regarding a group of frame
 
     contains the frames type and if we should provide a copy of those in the
-    outputfile or not
+    output file or not
 
-    :param url: url to access the frame group (bliss scan entry)
-    :type url: None, str, DataUrl. If provided as a string should fit a DataUrl
+    :param url: url to access the frame group (bliss scan entry). If provided as a string should fit a DataUrl
                or be the data path on the input file
-    :param frame_type: frame type (dark, projection ...)
-    :type param_type: AcquisitionStep, str. If provided as str should be
-                      an AcquisitionStep value
-    :param copy: should we copy the dataframe in the output file
-    :type copy: bool, None. If set to None then the value will be set to
-                the default copy behavior during processing
+    :param frame_type: frame type (dark, projection ...). If provided as str should be an AcquisitionStep value
+    :param copy: should we copy the data frame in the output file. If set to None then the value will be set to the default copy behavior during processing
     """
 
     class Info(_Enum):
@@ -68,9 +39,9 @@ class FrameGroup:
 
     def __init__(
         self,
-        url: Union[DataUrl, str, None],
-        frame_type: [AcquisitionStep, str],
-        copy: Union[bool, None] = None,
+        url: DataUrl | str | None,
+        frame_type: AcquisitionStep | str,
+        copy: bool | None = None,
     ):
         self._url = None
         self._frame_type = None
@@ -82,13 +53,13 @@ class FrameGroup:
         self.copy = copy
 
     @property
-    def copy(self) -> Union[bool, None]:
+    def copy(self) -> bool | None:
         return self._copy
 
     @copy.setter
     def copy(self, copy: bool):
         if not isinstance(copy, (bool, type(None))):
-            raise TypeError("copy should be a bool and not {}" "".format(type(copy)))
+            raise TypeError(f"copy should be a bool and not {type(copy)}")
         else:
             self._copy = copy
 
@@ -97,15 +68,15 @@ class FrameGroup:
         return self._frame_type
 
     @frame_type.setter
-    def frame_type(self, frame_type: Union[str, int, AcquisitionStep]):
+    def frame_type(self, frame_type: str | int | AcquisitionStep):
         self._frame_type = AcquisitionStep.from_value(frame_type)
 
     @property
-    def url(self) -> Union[DataUrl, None]:
+    def url(self) -> DataUrl | None:
         return self._url
 
     @url.setter
-    def url(self, url: Union[DataUrl, str]):
+    def url(self, url: DataUrl | str):
         if isinstance(url, str):
             # handle if the string path is not a "full url" then we consider
             # this is only the data path.
@@ -123,9 +94,7 @@ class FrameGroup:
         elif isinstance(url, (DataUrl, type(None))):
             self._url = url
         else:
-            raise TypeError(
-                "url should be a str or a DataUrl not {}" "".format(type(url))
-            )
+            raise TypeError(f"url should be a str or a DataUrl not {type(url)}")
 
     def __str__(self) -> str:
         return self.str_representation(
@@ -141,14 +110,9 @@ class FrameGroup:
         :param only_data_path: if True consider the input file frame group is
                                contained in the input file and the string
                                representing the url can be only the data path
-        :type only_data_path: bool
         :param with_copy: if true display the copy information
-        :type with_copy: bool
         :param with_prefix_key: if true provide the string with the keys as
                                 prefix (frame_type=XXX, copy=...)
-        :type with_prefix_key: bool
-
-        :rtype: str
         """
         if self.url is None:
             url_str = ""
@@ -195,13 +159,11 @@ class FrameGroup:
         if len(urls) == 0:
             return ""
         else:
-            urls_str = ",\n".join(["{}".format(str(url)) for url in urls])
-            return """(
-{}
+            urls_str = ",\n".join([f"{url}" for url in urls])
+            return f"""(
+{urls_str}
 )
-            """.format(
-                urls_str
-            )
+            """
 
     @staticmethod
     def frm_str(input_str: str):
@@ -211,16 +173,14 @@ class FrameGroup:
         if not isinstance(input_str, str):
             raise TypeError(f"{input_str} should be a string")
 
+        from nxtomomill.io.utils import filter_str_def  # avoid cyclic import
         from nxtomomill.io.utils import (
             remove_parenthesis_or_brackets,
         )  # avoid cyclic import
-        from nxtomomill.io.utils import filter_str_def  # avoid cyclic import
 
         def add_info(info_type, value):
             if info_type in cst_inputs:
-                err = "{} is provided twice: {} and {}".format(
-                    info_type.value, value, cst_inputs[info_type]
-                )
+                err = f"{info_type.value} is provided twice: {value} and {cst_inputs[info_type]}"
                 raise ValueError(err)
             else:
                 cst_inputs[info_type] = value
@@ -235,7 +195,7 @@ class FrameGroup:
             assert isinstance(elmt, str)
             # first try to get elements from the "INFO=" format
             for info in FrameGroup.Info.members():
-                key = "{}=".format(info.value)
+                key = f"{info.value}="
                 if elmt.startswith(key):
                     add_info(
                         info_type=info, value=filter_str_def(elmt.replace(key, "", 1))
@@ -253,14 +213,14 @@ class FrameGroup:
                 return
 
             # is this a copy element
-            if elmt in ("True", "False"):
+            if elmt.title() in ("True", "False"):
                 add_info(info_type=FrameGroup.Info.COPY, value=elmt)
                 return
 
             try:
                 elmt = filter_str_def(elmt)
                 DataUrl(path=elmt)
-            except Exception:
+            except ValueError:
                 pass
             else:
                 add_info(info_type=FrameGroup.Info.URL_ENTRY, value=elmt)
@@ -271,36 +231,34 @@ class FrameGroup:
                 data_path="/data/path",
                 scheme="h5py",
             )
+            _example_frame = FrameGroup(
+                url=url_example, copy=True, frame_type="projection"
+            )
             err_msg = (
                 "Unable to interpret string. Please insure this is a "
                 "either a frame type, a boolean for copy or an entry "
                 "(DataUrl).\n "
                 "Please prefix the value by the information type like: "
-                "{}. Invalid element is {}".format(
-                    FrameGroup(url=url_example, copy=True, frame_type="projection"),
-                    input_str,
-                )
+                f"{_example_frame}. Invalid element is {input_str}"
             )
             raise ValueError(err_msg)
 
         [treat_elmt(elmt) for elmt in elmts]
-
         # insure we have at least the frame type and the DataUrl
         inputs = {}
         if FrameGroup.Info.FRAME_TYPE not in cst_inputs:
-            raise ValueError("Unable to find frame type from {}".format(input_str))
+            raise ValueError(f"Unable to find frame type from {input_str}")
         else:
             inputs["frame_type"] = cst_inputs[FrameGroup.Info.FRAME_TYPE]
         if FrameGroup.Info.URL_ENTRY not in cst_inputs:
-            raise ValueError("Unable to find entry from {}".format(input_str))
+            raise ValueError(f"Unable to find entry from {input_str}")
         else:
             inputs["url"] = cst_inputs[FrameGroup.Info.URL_ENTRY]
         if FrameGroup.Info.COPY in cst_inputs:
             inputs["copy"] = cst_inputs[FrameGroup.Info.COPY]
 
-        if "copy" in inputs and inputs["copy"] in ("True", "False"):
-            inputs["copy"] = inputs["copy"] == "True"
-
+        if "copy" in inputs and inputs["copy"].title() in ("True", "False"):
+            inputs["copy"] = inputs["copy"].title() == "True"
         return FrameGroup(**inputs)
 
 
@@ -310,16 +268,16 @@ def filter_acqui_frame_type(
     """compute the list of urls representing projections
     from init until the next Initialization step
 
-    :param FrameGroup init: frame group creating the beginning of the
+    :param init: frame group creating the beginning of the
                             acquisition sequence
-    :param tuple sequences: list of FrameGroup representing the sequence
-    :param AcquisitionStep frame_type: type of frame to filer (cannot be Initialization step)
+    :param sequences: list of FrameGroup representing the sequence
+    :param frame_type: type of frame to filer (cannot be Initialization step)
     """
     frame_type = AcquisitionStep.from_value(frame_type)
     if frame_type is AcquisitionStep.INITIALIZATION:
-        raise ValueError("{} is not handled".format(AcquisitionStep.INITIALIZATION))
+        raise ValueError(f"{AcquisitionStep.INITIALIZATION.value} is not handled")
     if init not in sequences:
-        raise ValueError("{} cannot be find in the provided sequence".format(init))
+        raise ValueError(f"{init} cannot be find in the provided sequence")
 
     frame_types = [frm_grp.frame_type for frm_grp in sequences]
     current_acqui_idx = sequences.index(init)
