@@ -1,61 +1,46 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2019 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
+
 """An :class:`.Enum` class with additional features."""
 
-__authors__ = ["T. Vincent", "H. Payno"]
-__license__ = "MIT"
-__date__ = "29/04/2019"
-
-
-import typing
-from datetime import datetime
-import numpy
+from __future__ import annotations
+import logging
 import os
-from tomoscan.io import HDF5File
+from datetime import datetime
+
+import numpy
 from silx.io.url import DataUrl
 from silx.io.utils import get_data
-import logging
+from silx.io.utils import open as open_hdf5
 from silx.utils.deprecation import deprecated
-from tomoscan.esrf.scan.utils import cwd_context
-from .frameappender import FrameAppender
 from silx.utils.enum import Enum as _Enum
+from tomoscan.esrf.scan.utils import cwd_context
+from tomoscan.io import HDF5File
+
+from nxtomo.nxobject.nxdetector import ImageKey
+from nxtomo.utils.frameappender import FrameAppender
+from nxtomo.application.nxtomo import NXtomo
 
 try:
     import hdf5plugin  # noqa F401
 except ImportError:
     pass
-from collections.abc import Iterable
 import uuid
+from typing import Iterable
+
 from silx.io.utils import h5py_read_dataset
 
-try:
-    from tomoscan.esrf.scan.hdf5scan import ImageKey
-    from tomoscan.esrf.scan.hdf5scan import HDF5TomoScan
-except ImportError:
-    from tomoscan.esrf.hdf5scan import ImageKey
-    from tomoscan.esrf.hdf5scan import HDF5TomoScan
+
+__all__ = [
+    "embed_url",
+    "FileExtension",
+    "get_file_name",
+    "get_tuple_of_keys_from_cmd",
+    "is_nx_tomo_entry",
+    "add_dark_flat_nx_file",
+    "change_image_key_control",
+    "str_datetime_to_numpy_datetime64",
+    "strip_extension",
+]
 
 
 def embed_url(url: DataUrl, output_file: str) -> DataUrl:
@@ -64,11 +49,9 @@ def embed_url(url: DataUrl, output_file: str) -> DataUrl:
     to store it
 
     :param DataUrl url: dataset to be copied
-    :param str output_file: where to store the dataset
+    :param output_file: where to store the dataset
     :param expected_type: some metadata to put in copied dataset attributes
-    :type expected_type: Union[None, AcquisitionStep]
     :param data: data loaded from url is already loaded
-    :type data: None, numpy.array
     """
     if not isinstance(url, DataUrl):
         return url
@@ -91,27 +74,21 @@ class FileExtension(_Enum):
     NX = ".nx"
 
 
-class Format(_Enum):
-    STANDARD = "standard"
-    XRD_CT = "xrd-ct"
-    XRD_3D = "3d-xrd"
-
-
 def get_file_name(file_name, extension, check=True):
     """
     set the given extension
 
-    :param str file_name: name of the file
-    :param str extension: extension to give
-    :param bool check: if check, already check if the file as one of the
+    :param file_name: name of the file
+    :param extension: extension to give
+    :param check: if check, already check if the file as one of the
                        '_FileExtension'
     """
     if isinstance(extension, str):
-        extension = FileExtension.from_value(extension.lower())
+        extension = FileExtension(extension.lower())
     assert isinstance(extension, FileExtension)
     if check:
-        for value in FileExtension.values():
-            if file_name.lower().endswith(value):
+        for item in FileExtension:
+            if file_name.lower().endswith(item.value):
                 return file_name
     return file_name + extension.value()
 
@@ -124,29 +101,29 @@ def get_tuple_of_keys_from_cmd(cmd_value: str) -> tuple:
 def is_nx_tomo_entry(file_path, entry):
     """
 
-    :param str file_path: hdf5 file path
-    :param str entry: entry to check
+    :param file_path: hdf5 file path
+    :param entry: entry to check
     :return: True if the entry is an NXTomo entry
     """
     if not os.path.exists(file_path):
         return False
     else:
-        with HDF5File(file_path, mode="r") as h5s:
+        with open_hdf5(file_path) as h5s:
             if entry not in h5s:
                 return False
             node = h5s[entry]
-            return HDF5TomoScan.node_is_nxtomo(node)
+            return NXtomo.node_is_nxtomo(node)
 
 
 def add_dark_flat_nx_file(
     file_path: str,
     entry: str,
-    darks_start: typing.Union[None, numpy.ndarray, DataUrl] = None,
-    flats_start: typing.Union[None, numpy.ndarray, DataUrl] = None,
-    darks_end: typing.Union[None, numpy.ndarray, DataUrl] = None,
-    flats_end: typing.Union[None, numpy.ndarray, DataUrl] = None,
-    extras: typing.Union[None, dict] = None,
-    logger: typing.Union[None, logging.Logger] = None,
+    darks_start: numpy.ndarray | DataUrl | None = None,
+    flats_start: numpy.ndarray | DataUrl | None = None,
+    darks_end: numpy.ndarray | DataUrl | None = None,
+    flats_end: numpy.ndarray | DataUrl | None = None,
+    extras: dict | None = None,
+    logger: None | logging.Logger = None,
     embed_data: bool = False,
 ):
     """
@@ -173,18 +150,12 @@ def add_dark_flat_nx_file(
     `count_time` will be copied each time.
 
     :param file_path: NXTomo file containing data to be patched
-    :type file_path: str
     :param entry: entry to be patched
-    :type entry: str
     :param darks_start: (3D) numpy array containing the first dark serie if any
-    :type darks_start: Union[None, numpy.ndarray, DataUrl]
     :param flats_start: (3D) numpy array containing the first flat if any
-    :type flats_start: Union[None, numpy.ndarray, DataUrl]
     :param darks_end: (3D) numpy array containing dark the second dark serie if
                       any
-    :type darks_end: Union[None, numpy.ndarray, DataUrl]
     :param flats_end: (3D) numpy array containing the second flat if any
-    :type flats_end: Union[None, numpy.ndarray, DataUrl]
     :param extras: dictionary to specify some parameters for flats and dark
                    like rotation angle.
                    valid keys: 'start_dark', 'end_dark', 'start_flag',
@@ -194,9 +165,8 @@ def add_dark_flat_nx_file(
                    Possible values are:
                    * `count_time`
                    * `rotation_angle`
-    :type extras: Union[None, dict]
-    :param Union[None, logging.Logger] logger: object for logs
-    :param bool embed_data: if True then each external data will be copy
+    :param logger: object for logs
+    :param embed_data: if True then each external data will be copy
                             under a 'duplicate_data' folder
     """
     if extras is None:
@@ -206,8 +176,7 @@ def add_dark_flat_nx_file(
             valid_extra_keys = ("darks_start", "darks_end", "flats_start", "flats_end")
             if key not in valid_extra_keys:
                 raise ValueError(
-                    "{key} is not recognized. Valid values are "
-                    "{keys}".format(key=key, keys=valid_extra_keys)
+                    f"{key} is not recognized. Valid values are {valid_extra_keys}"
                 )
 
     if embed_data is True:
@@ -223,8 +192,7 @@ def add_dark_flat_nx_file(
                         raise ValueError(
                             "When data is not embed slice `step`"
                             "must be None or 1. Other values are"
-                            "not handled. Failing url is {}"
-                            "".format(url)
+                            f"not handled. Failing url is {url}"
                         )
 
     # !!! warning: order of dark / flat treatments import
@@ -242,27 +210,24 @@ def add_dark_flat_nx_file(
         if data is None:
             continue
         n_frames_to_insert = 1
+        if isinstance(data, str):
+            data = DataUrl(path=data)
         if isinstance(data, numpy.ndarray) and data.ndim == 3:
             n_frames_to_insert = data.shape[0]
         elif isinstance(data, DataUrl):
-            with HDF5File(data.file_path(), mode="r") as h5s:
+            with open_hdf5(data.file_path()) as h5s:
                 if data.data_path() not in h5s:
                     raise KeyError(
-                        "Path given ({}) is not in {}".format(
-                            data.data_path(), data.file_path
-                        )
+                        f"Path given ({data.data_path()}) is not in {data.file_path}"
                     )
-                data_node = get_data(data)
-                if data_node.ndim == 3:
-                    n_frames_to_insert = data_node.shape[0]
+            data_node = get_data(data)
+            if data_node.ndim == 3:
+                n_frames_to_insert = data_node.shape[0]
         else:
-            raise TypeError("{} as input is not managed".format(type(data)))
+            raise TypeError(f"{type(data)} as input is not managed")
 
         if logger is not None:
-            logger.info(
-                "insert {data_type} frame of type {key} at the"
-                "{where}".format(data_type=type(data), key=key, where=where)
-            )
+            logger.info(f"insert {type(data)} frame of type {key} at the {where}")
         # update 'data' dataset
         data_path = os.path.join(entry, "instrument", "detector", "data")
         FrameAppender(
@@ -290,15 +255,15 @@ def add_dark_flat_nx_file(
         )
         rotation_angle_path = os.path.join(entry, "sample", "rotation_angle")
         x_translation_path = os.path.join(entry, "sample", "x_translation")
-        y_translation_path = os.path.join(entry, "sample", "y_translation")
-        z_translation_path = os.path.join(entry, "sample", "z_translation")
+        translation_y_path = os.path.join(entry, "sample", "translation_y")
+        translation_z_path = os.path.join(entry, "sample", "translation_z")
         control_data_path = os.path.join(entry, "control", "data")
         data_key_paths = (
             count_time_path,
             rotation_angle_path,
             x_translation_path,
-            y_translation_path,
-            z_translation_path,
+            translation_y_path,
+            translation_z_path,
             control_data_path,
         )
         mandatory_keys = (
@@ -307,8 +272,8 @@ def add_dark_flat_nx_file(
         )
         optional_keys = (
             "x_translation",
-            "y_translation",
-            "z_translation",
+            "translation_y",
+            "translation_z",
             "control/data",
         )
 
@@ -322,9 +287,8 @@ def add_dark_flat_nx_file(
                     if len(provided_value) != n_frames_to_insert:
                         raise ValueError(
                             "Given value to store from extras has"
-                            " incoherent length({}) compare to "
-                            "the number of frame to save ({})"
-                            "".format(len(provided_value), n_frames_to_insert)
+                            f" incoherent length({len(provided_value)}) compare to "
+                            f"the number of frame to save ({n_frames_to_insert})"
                         )
                     else:
                         data_to_insert = provided_value
@@ -332,12 +296,12 @@ def add_dark_flat_nx_file(
                     try:
                         data_to_insert = [provided_value] * n_frames_to_insert
                     except Exception as e:
-                        logger.error("Fail to create data to insert. Error is", e)
+                        logger.error(f"Fail to create data to insert. Error is {e}")
                         return
             else:
                 # get default values
                 def get_default_value(location, where_):
-                    with HDF5File(file_path, mode="r") as h5s:
+                    with open_hdf5(file_path) as h5s:
                         if location not in h5s:
                             return None
                         existing_data = h5s[location]
@@ -351,10 +315,7 @@ def add_dark_flat_nx_file(
                 except Exception:
                     default_value = None
                 if default_value is None:
-                    msg = (
-                        "Unable to define a default value for {}. Location "
-                        "empty in {}.".format(data_key_path, file_path)
-                    )
+                    msg = f"Unable to define a default value for {data_key_path}. Location empty in {file_path}"
                     if data_key in mandatory_keys:
                         raise ValueError(msg)
                     elif logger:
@@ -362,9 +323,7 @@ def add_dark_flat_nx_file(
                     continue
                 elif logger:
                     logger.debug(
-                        "No value(s) provided for {path}. "
-                        "Extract some default value ({def_value})."
-                        "".format(path=data_key_path, def_value=default_value)
+                        f"No value(s) provided for {data_key_path}. Extract some default value ({default_value})."
                     )
                 data_to_insert = [default_value] * n_frames_to_insert
 
@@ -403,21 +362,19 @@ def _insert_frame_data(data, file_path, data_path, where, logger=None):
 def change_image_key_control(
     file_path: str,
     entry: str,
-    frames_indexes: typing.Union[slice, Iterable],
-    image_key_control_value: typing.Union[int, ImageKey],
+    frames_indexes: slice | Iterable,
+    image_key_control_value: int | ImageKey,
     logger=None,
 ):
     """
     Will modify image_key and image_key_control values for the requested
     frames.
 
-    :param str file_path: path the nexus file
-    :param str entry: name of the entry to modify
+    :param file_path: path the nexus file
+    :param entry: name of the entry to modify
     :param frames_indexes: index of the frame for which we want to modify
                            the image key
-    :type frames_indexes: Union[slice, Iterable]
     :param image_key_control_value:
-    :type image_key_control_value: Union[int, ImageKey]
     :param logging.Logger logger: logger
     """
     if not isinstance(frames_indexes, (Iterable, slice)):
@@ -434,7 +391,7 @@ def change_image_key_control(
             )
         )
 
-    image_key_control_value = ImageKey.from_value(image_key_control_value)
+    image_key_control_value = ImageKey(image_key_control_value)
     with HDF5File(file_path, mode="a") as h5s:
         node = h5s[entry]
         image_keys_path = "/".join(("instrument", "detector", "image_key"))
@@ -467,9 +424,7 @@ def change_image_key_control(
         node[image_keys_path][:] = image_keys
 
 
-def str_datetime_to_numpy_datetime64(
-    my_datetime: typing.Union[str, datetime]
-) -> numpy.datetime64:
+def str_datetime_to_numpy_datetime64(my_datetime: str | datetime) -> numpy.datetime64:
     # numpy deprecates time zone awarness conversion to numpy.datetime64.
     # so we remove the time zone info.
     if isinstance(my_datetime, str):
